define([
         "util/classutils",
         "bionic/in_game_screens/pause"
         ],
         function(classUtils, Pause) {
	
	"use strict";
	
	return classUtils.create({
		canvas:null,
		holder:null,
		
		pause:null,
		
		character_left:null,
		character_right:null,
		ingame_character:null,
		
		bucket_array:[],
		dropped_items_array:[],
		items_collected:0,
		selected_item_data:"",
		selected_item:null,
		drop_amount:[5, 7, 8, 10, 10],
		
		//start_positions:[{x:-150, y:437}, {x:1100, y:437}],
		//end_positions:[{x:125, y:437}, {x:819, y:437}],
		
		character_left_positions:null,
		character_right_positions:null,
		
		well_done_text:null,
		
		pause_btn:null,
		sound_btn:null,
		
		pause_active:false,
		
		pause_container:null,
		pause_array:[],
		
		swap_array:[0,0,5,3,2],
		swap_timer:null,
		switching:false,
		
		clock:null,
		clock_segment_array:null,
		clock_update_timer:null,
		current_time:0,
		
		spark_positions:[{x:377, y:71}, {x:447, y:62}, {x:569, y:61}, {x:568, y:155}, {x:582, y:238}, {x:361, y:249}, {x:356, y:202}],
		spark_timer:null,
				
		init: function()
		{
			var self = this;
						
			self.canvas = new createjs.Stage("game_canvas");
						
			self.holder = new createjs.Container();
			self.canvas.addChild(self.holder);
			
			var background = window.game.sprites.get_bitmap("preloader_background");
			self.holder.addChild(background);
			
			var graphic, image_data;
			
			self.character_left = new createjs.Container();
			//self.character_left.x = 125;
			//self.character_left.y = 437;
			self.holder.addChild(self.character_left);
			
			graphic = window.game.sprites.get_bitmap("character_left");
			image_data = window.game.sprites.get_data("character_left");
			graphic.x = 0 - (image_data.width/2);
			graphic.y = 0 - (image_data.height/2);
			self.character_left.addChild(graphic);
			
			self.character_left_positions = {
												start:{x:-(image_data.width/2), y:window.gameHeight - (image_data.height/2)}, 
												end:{x:(image_data.width/2), y:window.gameHeight - (image_data.height/2)}
											};
			
			self.ingame_character = new createjs.Container();
			self.ingame_character.x = window.gameWidth;
			self.ingame_character.y = window.gameHeight;
			self.holder.addChild(self.ingame_character);
			
			graphic = window.game.sprites.get_bitmap("ingame_character");
			image_data = window.game.sprites.get_data("ingame_character");
			graphic.x = -image_data.width;
			graphic.y = -image_data.height;
			self.ingame_character.addChild(graphic);
						
			self.character_right = new createjs.Container();
			//self.character_right.x = 819;
			//self.character_right.y = 437;
			self.holder.addChild(self.character_right);
			
			graphic = window.game.sprites.get_bitmap("character_right");
			image_data = window.game.sprites.get_data("character_right");
			graphic.x = 0 - (image_data.width/2);
			graphic.y = 0 - (image_data.height/2);
			self.character_right.addChild(graphic);
			
			self.character_right_positions = {
												start:{x:window.gameWidth+(image_data.width/2), y:window.gameHeight - (image_data.height/2)}, 
												end:{x:window.gameWidth-(image_data.width/2), y:window.gameHeight - (image_data.height/2)}
											};
			
			self.selected_item_data = {type:"", xPos:0,yPos:0,name:""};
			
			self.clock = new createjs.Container();
			self.clock.x = window.gameWidth/2;
			self.clock.y = 155;
			self.holder.addChild(self.clock);
			
			var circle = new createjs.Shape();
			circle.y = 10;
			circle.graphics.beginFill("#ffde00");
			circle.graphics.drawCircle(0,0,50,50);
			circle.graphics.endFill();
			self.clock.addChild(circle);
			
			var pieces_in_circle = 60;
			var rot_increment = 6;
			
			self.clock_segment_array = [];
			
			for (var i=0; i<pieces_in_circle; i++)
			{
				var blue_tri = new createjs.Shape();
				blue_tri.y = 10;
				blue_tri.graphics.beginFill("#58b0e3");
				blue_tri.graphics.moveTo(0, -60).lineTo(12, -60).lineTo(0, 0);
				blue_tri.graphics.endFill();
				blue_tri.rotation = rot_increment * i;
				blue_tri.visible = false;
				self.clock.addChild(blue_tri);
				
				var mask = new createjs.Shape();
				mask.y = 10;
				mask.graphics.beginFill("#ffde00");
				mask.graphics.drawCircle(0,0,50,50);
				mask.graphics.endFill();
				blue_tri.mask = mask;
				
				self.clock_segment_array.push(blue_tri);
			}
			
			self.clock.scaleX = self.clock.scaleY = 0;
			
			var overlay_graphic, image_data;
			overlay_graphic = window.game.sprites.get_bitmap("default_assets", "clock_overlay");
			image_data = window.game.sprites.get_data("default_assets", "clock_overlay");
			overlay_graphic.x = 0 - (image_data.width/2);
			overlay_graphic.y = 0 - (image_data.height/2);
			self.clock.addChild(overlay_graphic);
			
			self.pause = new Pause(self.holder);
		},
		
		show : function ()
		{
			var self = this;	
			self.holder.visible = true;
			TweenMax.to(self.holder, 0.5, {alpha:1});
			
			window.game.sound_managers.bgm.play("game");
			
			self.build_page();
			
			dax_event('hidden','','&c5_gm_action=started');
			console.log('event: start');
		},
		
		show_active_game_from_instructions : function ()
		{
			var self = this;
			
			self.holder.visible = true;
			TweenMax.to(self.holder, 0.5, {alpha:1});
			
			self.pause.add_clicks(1);	
		},
		
		hide : function ()
		{
			var self = this;
			
			TweenMax.to(self.holder, 0.35, {alpha:0, onComplete:function(){
				self.holder.visible = false;
			}});
			
			self.remove_ui();
		},
		
		build_ui : function(){
			var self = this;
			self.pause_btn = new createjs.Container();
			self.pause_btn.x = 889;
			self.pause_btn.y = 55;
			self.holder.addChild(self.pause_btn);
			
			var graphic, image_data;
						
			graphic = window.game.sprites.get_bitmap("default_assets","ui_pause");
			image_data = window.game.sprites.get_data("default_assets","ui_pause");
			graphic.x = 0 - (image_data.width/2);
			graphic.y = 0 - (image_data.height/2);
			self.pause_btn.addChild(graphic);
			
			self.sound_btn = new createjs.Container();
			self.sound_btn.x = 802;
			self.sound_btn.y = 55;
			self.holder.addChild(self.sound_btn);
						
			var btn_type;
			
			if (window.game.sound_on == true)
			{
				btn_type = "ui_sound_on";
			} else {
				btn_type = "ui_sound_off";
			}
						
			graphic = window.game.sprites.get_bitmap("default_assets",btn_type);
			image_data = window.game.sprites.get_data("default_assets",btn_type);
			graphic.x = 0 - (image_data.width/2);
			graphic.y = 0 - (image_data.height/2);
			self.sound_btn.addChild(graphic);
			
			self.add_ui_click();
		},
		
		remove_ui : function()
		{
			var self = this;
			
			self.holder.removeChild(self.pause_btn);
			self.holder.removeChild(self.sound_btn);
			
			self.remove_ui_click();
		},
		
		add_ui_click:function()
		{
			var self = this;
			var image_data = window.game.sprites.get_data("default_assets","ui_pause");
			
			var rect = {x:self.pause_btn.x - (image_data.width/2), y:self.pause_btn.y - (image_data.height/2), w:image_data.width, h:image_data.height};
			window.game.mouse.add_click('game_pause_btn_click', rect, function(){
				self.pause_game();
				self.pause.show();
			});
			
			window.game.size_adjuster.add_item(self.pause_btn, rect, -1, 1);
			
			image_data = window.game.sprites.get_data("default_assets","ui_sound_on");
			
			rect = {x:self.sound_btn.x - (image_data.width/2), y:self.sound_btn.y - (image_data.height/2), w:image_data.width, h:image_data.height};
			window.game.mouse.add_click('game_sound_btn_click', rect, function(){
				self.sound_toggle();
			});
			
			window.game.size_adjuster.add_item(self.sound_btn, rect, -1, 1);	
		},
		
		remove_ui_click : function()
		{
			var self = this;	
			window.game.mouse.remove_click("game_pause_btn_click");
			window.game.mouse.remove_click("game_sound_btn_click");
		},
		
		pause_game : function()
		{
			var self = this;
			self.pause_active = true;
			self.remove_ui();	
			self.pause_clock();
			clearTimeout(self.swap_timer);
			clearInterval(self.spark_timer);
			TweenMax.killTweensOf(self.holder);	
			
			self.reset_listeners();
			
			self.clock.visible = false;
			for (var i=0; i < self.bucket_array.length; i++)
			{
				self.bucket_array[i].container.visible = false;
				self.bucket_array[i].inside.visible = false;
			}
		},
		
		unpause_game : function()
		{
			var self = this;	
			self.build_ui();
			self.start_swap_timer();
			self.unpause_clock();
			setTimeout(function(){self.pause_active = false;}, 250);
			
			self.add_listeners();
			
			self.clock.visible = true;
			for (var i=0; i < self.bucket_array.length; i++)
			{
				self.bucket_array[i].container.visible = true;
				self.bucket_array[i].inside.visible = true;
			}
		},
		
		restart_game:function()
		{
			var self = this;	
			window.game.current_level = 1;
			self.clean_up_stage();
			self.build_page();
			setTimeout(function(){self.pause_active = false;}, 250);
		},
		
		sound_toggle : function()
		{
			var self = this;
			
			self.holder.removeChild(self.sound_btn);
			
			self.sound_btn = new createjs.Container();
			self.sound_btn.x = 802;
			self.sound_btn.y = 55;
			self.holder.addChild(self.sound_btn);
			
			var btn_type;
			
			if (window.game.sound_on == true)
			{
				window.game.sound_on = false;
				btn_type = "ui_sound_off";
				window.game.sound_managers.bgm.mute();
			} else {
				window.game.sound_on = true;
				btn_type = "ui_sound_on";
				window.game.sound_managers.bgm.unmute();
			}
			
			var graphic, image_data;
						
			graphic = window.game.sprites.get_bitmap("default_assets",btn_type);
			image_data = window.game.sprites.get_data("default_assets",btn_type);
			graphic.x = 0 - (image_data.width/2);
			graphic.y = 0 - (image_data.height/2);
			self.sound_btn.addChild(graphic);
			
			window.game.size_adjuster.add_item(self.sound_btn, null, -1, 1);	
		},
		
		build_page: function(new_level)
		{
			var self = this;
			
			var title_text = new createjs.Container();
			title_text.x = 474;
			title_text.y = 150;
			self.holder.addChild(title_text);
			
			var graphic, image_data;
						
			graphic = window.game.sprites.get_bitmap("text_level_" + window.game.current_level);
			image_data = window.game.sprites.get_data("text_level_" + window.game.current_level);
			graphic.x = 0 - (image_data.width/2);
			graphic.y = 0 - (image_data.height/2);
			title_text.addChild(graphic);
			
			title_text.scaleX = title_text.scaleY = 0;
			TweenMax.to(title_text, 0.25, {delay:0.5, scaleX:1, scaleY:1, ease:Back.easeOut, onComplete:function(){
				TweenMax.to(title_text, 0.35, {delay:1, alpha:0, onComplete:function(){
					self.holder.removeChild(title_text);
				}});
			}});
			
			if (new_level == false)
			{
				/*
				self.character_left.x = self.start_positions[0].x;
				self.character_left.y = self.start_positions[0].y;
				self.character_right.x = self.start_positions[1].x;
				self.character_right.y = self.start_positions[1].y;
			
				TweenMax.to(self.character_left, 0.15, {delay:0.25, ease:Linear.easeOut, x:self.end_positions[0].x, y:self.end_positions[0].y, onComplete:function(){
					TweenMax.to(self.character_left, 0.25, {delay:0.75, ease:Linear.easeIn, x:self.start_positions[0].x, y:self.start_positions[0].y});
				}});
				TweenMax.to(self.character_right, 0.15, {delay:0.25, ease:Linear.easeOut, x:self.end_positions[1].x, y:self.end_positions[1].y, onComplete:function(){
					TweenMax.to(self.character_right, 0.25, {delay:0.75, ease:Linear.easeIn, x:self.start_positions[1].x, y:self.start_positions[1].y});
				}});*/
				
				self.character_left.x = self.character_left_positions.start.x;
				self.character_left.y = self.character_left_positions.start.y;
				self.character_right.x = self.character_right_positions.start.x;
				self.character_right.y = self.character_right_positions.start.y;
				
				TweenMax.to(self.character_left, 0.15, {delay:0.25, ease:Linear.easeOut, x:self.character_left_positions.end.x, y:self.character_left_positions.end.y, onComplete:function(){
					TweenMax.to(self.character_left, 0.25, {delay:0.75, ease:Linear.easeIn, x:self.character_left_positions.start.x, y:self.character_left_positions.start.y});
				}});
				TweenMax.to(self.character_right, 0.15, {delay:0.25, ease:Linear.easeOut, x:self.character_right_positions.end.x, y:self.character_right_positions.end.y, onComplete:function(){
					TweenMax.to(self.character_right, 0.25, {delay:0.75, ease:Linear.easeIn, x:self.character_right_positions.start.x, y:self.character_right_positions.start.y});
				}});
					
			} else {
				/*
				self.character_left.x = self.end_positions[0].x;
				self.character_left.y = self.end_positions[0].y;
				self.character_right.x = self.end_positions[1].x;
				self.character_right.y = self.end_positions[1].y;
			
				TweenMax.to(self.character_left, 0.25, {delay:0.75, ease:Linear.easeIn, x:self.start_positions[0].x, y:self.start_positions[0].y});
				TweenMax.to(self.character_right, 0.25, {delay:0.75, ease:Linear.easeIn, x:self.start_positions[1].x, y:self.start_positions[1].y});
				*/
				
				self.character_left.x = self.character_left_positions.end.x;
				self.character_left.y = self.character_left_positions.end.y;
				self.character_right.x = self.character_right_positions.end.x;
				self.character_right.y = self.character_right_positions.end.y;
				
				TweenMax.to(self.character_left, 0.25, {delay:0.75, ease:Linear.easeIn, x:self.character_left_positions.start.x, y:self.character_left_positions.start.y});
				TweenMax.to(self.character_right, 0.25, {delay:0.75, ease:Linear.easeIn, x:self.character_right_positions.start.x, y:self.character_right_positions.start.y});
			}
			
			TweenMax.to(self.ingame_character, 0.5, {delay:1.25, ease:Linear.easeIn, onComplete:function(){
				self.start_level();
			}});
			
			self.setup_buckets();
		},
		
		start_level : function()
		{
			var self = this;	
			self.items_collected = 0;
			self.current_time = 0;
			self.selected_item_data = {type:"", xPos:0,yPos:0,name:""};
			self.drop_items();
			self.pause_active = false;
			self.time_passed = 0;
			TweenMax.killTweensOf(self.holder);
			
			self.clock.visible = true;
			self.clock.scaleX = self.clock.scaleY = 0;
			self.clock.rotation = 0;
			self.reset_clock();
			TweenMax.to(self.clock, 0.25, {scaleX:1, scaleY:1, ease:Back.easeOut});	
			
			self.start_swap_timer();
			
		},
		
		setup_buckets:function(){
			var self = this;
			var reduced_label_scale = 0.65;
			var bucket_data_array = [
					[{type:'game_object_2', x:406, y:469, label_scale:reduced_label_scale}, {type:'game_object_1', x:536, y:469, label_scale:reduced_label_scale}],
					[{type:'game_object_3', x:343, y:469, label_scale:reduced_label_scale}, {type:'game_object_2', x:473, y:469, label_scale:reduced_label_scale}, {type:'game_object_1', x:603, y:469, label_scale:reduced_label_scale}],
					[{type:'game_object_3', x:343, y:469, label_scale:reduced_label_scale}, {type:'game_object_2', x:473, y:469, label_scale:reduced_label_scale}, {type:'game_object_1', x:603, y:469, label_scale:reduced_label_scale}],
					[{type:'game_object_3', x:277, y:469, label_scale:reduced_label_scale}, {type:'game_object_2', x:407, y:469, label_scale:reduced_label_scale}, {type:'game_object_1', x:537, y:469, label_scale:reduced_label_scale}, {type:'game_object_4', x:667, y:469, label_scale:reduced_label_scale}],
					[{type:'game_object_3', x:277, y:469, label_scale:reduced_label_scale}, {type:'game_object_2', x:407, y:469, label_scale:reduced_label_scale}, {type:'game_object_1', x:537, y:469, label_scale:reduced_label_scale}, {type:'game_object_4', x:667, y:469, label_scale:reduced_label_scale}]
			];
			
			var level_array = bucket_data_array[window.game.current_level - 1];
			
			self.bucket_array = [];
			
			for (var i=0; i < level_array.length; i++)
			{
				var bucket_inside = new createjs.Container();
				bucket_inside.x = level_array[i].x;
				bucket_inside.y = level_array[i].y - 45;
				self.holder.addChild(bucket_inside);
				
				var graphic, image_data;
				
				graphic = window.game.sprites.get_bitmap("default_assets", "bucket_inside");
				image_data = window.game.sprites.get_data("default_assets", "bucket_inside");
				graphic.x = 0 - (image_data.width/2);
				graphic.y = 0 - (image_data.height/2);
				bucket_inside.addChild(graphic);
				
				var bucket = new createjs.Container();
				bucket.x = level_array[i].x;
				bucket.y = level_array[i].y;
				self.holder.addChild(bucket);
				
				self.bucket_array.push({container:bucket, item:level_array[i].type, inside:bucket_inside});
				
				graphic = window.game.sprites.get_bitmap("default_assets", "bucket");
				image_data = window.game.sprites.get_data("default_assets", "bucket");
				graphic.x = 0 - (image_data.width/2);
				graphic.y = 0 - (image_data.height/2);
				bucket.addChild(graphic);
				
				var label = new createjs.Container();
				label.x = 0;
				label.y = 10;
				label.scaleX = label.scaleY = level_array[i].label_scale;
				bucket.addChild(label);
				
				graphic = window.game.sprites.get_bitmap(level_array[i].type);
				image_data = window.game.sprites.get_data(level_array[i].type);
				graphic.x = 0 - (image_data.width/2);
				graphic.y = 0 - (image_data.height/2);
				label.addChild(graphic);
			}	
		},
		
		drop_items : function()
		{
			var self = this;
			
			var available_items = [["game_object_1", "game_object_2"], ["game_object_1", "game_object_2", "game_object_3"], ["game_object_1", "game_object_2", "game_object_3"], ["game_object_1", "game_object_2", "game_object_3", "game_object_4"], ["game_object_1", "game_object_2", "game_object_3", "game_object_4"]];
			var item_counter = 0;
			var current_level = window.game.current_level;
			
			var previous_element;
			
			for (var i=0; i < self.drop_amount[current_level-1]; i++)
			{
				var rnd_x = window.game.maths.rand_int(150,600);
				var rnd_y = window.game.maths.rand_int(285,345);
				var rnd_time = window.game.maths.rand(0.5,1.5);
				var rnd_delay = window.game.maths.rand(0,1.5);
				var rnd_disperse = window.game.maths.rand_int(-100,100); 
				var rnd_item_index = previous_element;
				while (rnd_item_index == previous_element)
				{
					rnd_item_index = window.game.maths.rand_int(0,available_items[current_level-1].length-1);	
				}
				previous_element = rnd_item_index;
				var type = available_items[rnd_item_index];
				
				var item_container = new createjs.Container();
				item_container.x = rnd_x;
				item_container.y = rnd_y - 450;
				item_container.name = available_items[current_level-1][rnd_item_index];	
				self.holder.addChildAt(item_container, 1);

				var graphic = window.game.sprites.get_bitmap(available_items[current_level-1][rnd_item_index]);
				var image_data = window.game.sprites.get_data(available_items[current_level-1][rnd_item_index]);
				
				graphic.x = 0 - (image_data.width/2);
				graphic.y = 0 - (image_data.height/2);
				item_container.addChild(graphic);
				
				self.dropped_items_array.push(item_container);
				
				/*
setTimeout(function(){
					window.game.sound_managers[available_items[current_level-1][rnd_item_index]].play("drop");	
				}, (rnd_time) * 1000);
*/
				
				/* drop */
				TweenMax.to(item_container, rnd_time, {x:rnd_x,y:rnd_y,delay:rnd_delay,ease:Bounce.easeOut,onComplete:function(){
					item_counter++;
					if (item_counter == (self.drop_amount[current_level-1]))
					{
						self.add_listeners();
						self.build_ui();
					}
				}});
				
				//Add slight bounce off to an angle - good disperse effect
				TweenMax.to(item_container,0.5, {x:(rnd_x+rnd_disperse),delay:rnd_delay+0.4,ease:Circ.easeOut});
			}
		},
		
		add_listeners:function()
		{
			var self = this;
						
			for (var i = 0; i < self.dropped_items_array.length; i++) 
				{
					var object = self.dropped_items_array[i];
					var item_data = window.game.sprites.get_data(object.name);
					var rect = {x:object.x - (item_data.width/2), y:object.y - (item_data.height/2), w:item_data.width, h:item_data.height};
					
					window.game.mouse.add_click('drag_object_' + i, rect, function(data){
						if (self.selected_item_data.name == "" && self.pause_active == false)
						{
							self.selected_item = data.object;
							
							if (window.isMobile == true)
							{
								window.game.mouse.mouse_position.x = self.selected_item.x; 
								window.game.mouse.mouse_position.y = self.selected_item.y;
							}
							
							self.holder.setChildIndex(self.selected_item,(self.holder.getNumChildren()-1));
							self.selected_item.alpha = 1;
							
							self.selected_item_data.type = self.selected_item.name;
							self.selected_item_data.xPos = self.selected_item.x;
							self.selected_item_data.yPos = self.selected_item.y;
							self.selected_item_data.name = self.selected_item.name;
							self.add_stage_listener();
						}
					}, {object:object});	
				}

		},
		
		reset_listeners : function ()
		{
			var self = this;
			
			for (var i = 0; i < self.dropped_items_array.length; i++) 
			{
				window.game.mouse.remove_click('drag_object_' + i);
			}
		},
		
		add_stage_listener:function()
		{	
			var self = this;
			
			var rect_stage = {x:0, y:0, w:window.gameWidth, h:window.gameHeight};
		    
			window.game.mouse.add_click('game_click', rect_stage, function(){ 
				 if (self.selected_item_data.name != "")
				 {
					 self.selected_item_data = {type:"", xPos:0,yPos:0,name:""};
					 self.remove_stage_listener();
					 self.selected_item = null;
					 self.reset_listeners();
					 self.add_listeners();
					 /*
				 	var dragging_item = self.selected_item;
				 	TweenMax.to(dragging_item,0.5, {y:self.selected_item_data.yPos,x:self.selected_item_data.xPos,   
 						onComplete:function()
 							{
 							 self.selected_item_data = {type:"", xPos:0,yPos:0,name:""};
 							 self.remove_stage_listener();
 							}
 					
 					
 					});
 					*/
				 } 
			});
		},
		
		remove_stage_listener:function()
		{
			var self = this;
			  
			window.game.mouse.remove_click('game_click');
		},
		
		check_position:function()
		{
			var self = this;
			var image_data = window.game.sprites.get_data("default_assets", "bucket");
			var bucket_width = image_data.width;
			var bucket_height = image_data.height + 65;
			
			if (self.switching == false)
			{
				for (var i=0; i < self.bucket_array.length; i++)
				{
					if (self.selected_item.x >= self.bucket_array[i].container.x - (bucket_width/2) && self.selected_item.x <= self.bucket_array[i].container.x + (bucket_width/2))
					{
						if (self.selected_item.y >= self.bucket_array[i].container.y - (bucket_height/2) && self.selected_item.y <= self.bucket_array[i].container.y)
						{
							if (self.selected_item_data.name == self.bucket_array[i].item)
							{
								self.object_in_bucket(self.bucket_array[i].container, self.bucket_array[i].inside);
								window.game.sound_managers.sorting_game.play("correct");
							} else {
								self.reset_object();
								window.game.sound_managers.sorting_game.play("incorrect");
							}
						}
					}		
				}	
			}
		},
		
		reset_object : function()
		{
			var self = this;	
			TweenMax.to(self.selected_item, 0.15, {x:self.selected_item_data.xPos, y:self.selected_item_data.yPos, ease:Linear.easeNone});
			self.selected_item_data = {type:"", xPos:0,yPos:0,name:""};
			self.remove_stage_listener();
		},
		
		object_in_bucket:function(correct_bucket, bucket_inside)
		{
			var self = this;
			self.reset_listeners();
			self.selected_item_data = {type:"", xPos:0,yPos:0,name:""};
			self.holder.setChildIndex(self.selected_item, self.holder.getChildIndex(bucket_inside) + 1);
			self.selected_item.x = correct_bucket.x;
			self.selected_item.y = self.selected_item.y - 50;
			TweenMax.to(self.selected_item, 0.25, {x:correct_bucket.x, y:correct_bucket.y, alpha:1, onComplete:function(){
				for (var i = 0; i < self.dropped_items_array.length; i++) 
				{
					if (self.dropped_items_array[i] == self.selected_item)
					{
						self.dropped_items_array.splice(i, 1);
						break;
					}
				}
				self.holder.removeChild(self.selected_item);
				self.selected_item = null;
				self.remove_stage_listener();
				self.items_collected ++;
				self.check_end_of_level();
				self.add_listeners();
			}});
		},
		
		start_swap_timer : function()
		{
			var self = this;	
			if (self.swap_array[window.game.current_level - 1] != 0)
			{
				TweenMax.to(self.holder, self.swap_array[window.game.current_level-1], {onComplete:function(){
					self.swap_buckets();
				}});
			}
		},
		
		swap_buckets : function()
		{
			var self = this;
			var bucket_position_array = [];
			var switch_speed = 0.25;
			self.switching = true;
			
			window.game.sound_managers.sorting_game.play("bucket_switch");
			
			for (var i=0; i < self.bucket_array.length; i++)
			{
				bucket_position_array[i] = self.bucket_array[i].container.x;
			}
			
			for (var j=0; j < self.bucket_array.length; j++)
			{
				var random_element = Math.floor(Math.random() * bucket_position_array.length);
				var random_position = bucket_position_array[random_element];
				
				TweenMax.to(self.bucket_array[j].container, switch_speed, {x:random_position, ease:Linear.easeOut});
				TweenMax.to(self.bucket_array[j].inside, switch_speed, {x:random_position, ease:Linear.easeOut});
					
				if (j == self.bucket_array.length-1)
				{
					TweenMax.to(self.holder, switch_speed, {onComplete:function(){
						if (self.pause_active == false) {
							self.start_swap_timer();
							self.switching = false;	
						}
					}});
				}
				
				bucket_position_array.splice(random_element, 1);
			}
		},
		
		start_game_timer : function ()
		{
			var self = this;
			self.game_timer_loop();
		},
		
		start_clock : function()
		{
			var self = this;
			
			self.clock_update_timer = setInterval(function(){
				if (self.clock_segment_array[self.current_time] != null)
				{
					self.clock_segment_array[self.current_time].visible = true;
					self.current_time ++;
					
					if (window.isMobile == false)
					{
						if (self.current_time == self.clock_segment_array.length - 10)
						{
							self.shake_clock();
							self.start_sparks();
						}	
					}
				} else {
					clearInterval(self.clock_update_timer);
					TweenMax.killTweensOf(self.clock);
					if (window.isMobile == true)
					{
						self.clock.scaleX = self.clock.scaleY = 0;
						self.clock.visible = false;
						window.game.sound_managers.redemption.play("ring");
						self.level_fail();
						
					} else {
						TweenMax.to(self.clock, 0.25, {scaleY:0, scaleX:0, ease:Linear.easeIn, rotation:180, onComplete:function(){
							self.clock.visible = false;
							window.game.sound_managers.redemption.play("ring");
							self.level_fail();
						}});	
					}
				}
			}, 1000);
		},
		
		reset_clock : function()
		{
			var self = this;	
			self.current_time = 0;
			clearInterval(self.clock_update_timer);
			for (var i=0; i < self.clock_segment_array.length; i++)
			{
				self.clock_segment_array[i].visible = false;
			}
			
			self.clock_update_timer = null;
			self.start_clock();
		},
		
		pause_clock : function()
		{
			var self = this;
			clearInterval(self.clock_update_timer);	
			window.game.sound_managers.timer_tick.stop("tick");
		},
		
		unpause_clock:function()
		{
			var self = this;
			self.clock_update_timer = null;
			self.start_clock();	
			if (self.current_time == self.clock_segment_array.length - 10)
			{
				window.game.sound_managers.timer_tick.play("tick");
			}	
		},
		
		shake_clock: function()
		{
			var self = this;
			TweenMax.to(self.clock,0.075, {rotation:-5, ease:Linear.easeNone, onComplete:function(){
				TweenMax.to(self.clock, 0.15, {rotation:5, ease:Linear.easeNone, repeat:-1, yoyo:true});	
			}});
		},
		
		start_sparks : function()
		{
			var self = this;
			window.game.sound_managers.timer_tick.play("tick");
			var spark_speed = 0.5;
			self.spark_timer = setInterval(function(){
				self.generate_spark();	
			}, spark_speed * 1000);
		},
		
		generate_spark : function(){
			var self = this;
			var random_spark = Math.floor(Math.random() * self.spark_positions.length);
			var random_scale = Math.random();
			var random_rotation = Math.random() * 60;
			var new_spark = new createjs.Container();
			new_spark.x = self.spark_positions[random_spark].x;
			new_spark.y = self.spark_positions[random_spark].y;
			new_spark.scaleX = new_spark.scaleY = random_scale;
			new_spark.rotation = random_rotation;
			self.holder.addChild(new_spark);
			
			var graphic = window.game.sprites.get_bitmap("default_assets", "clock_spark");
			var image_data = window.game.sprites.get_data("default_assets", "clock_spark");
			graphic.x = 0 - (image_data.width/2);
			graphic.y = 0 - (image_data.height/2);
			new_spark.addChild(graphic);
			
			TweenMax.from(new_spark,0.15, {scaleX:0, scaleY:0, ease:Back.easeOut, onComplete:function(){
				TweenMax.to(new_spark, 0.1, {alpha:0, delay:0.25, onComplete:function(){
					self.holder.removeChild(new_spark);
				}});
			}});	
		},
		
		check_end_of_level:function()
		{
			var self = this;	
			
			if (self.items_collected == self.drop_amount[window.game.current_level-1])
			{
				window.game.sound_managers.timer_tick.stop("tick");
				TweenMax.to(self.ingame_character, 0.5, {ease:Linear.easeIn, onComplete:function(){
					self.level_complete();
				}});
			}
		},
		
		level_fail : function ()
		{
			var self = this;	
			
			window.game.sound_managers.timer_tick.stop("tick");
			
			self.pause_active = true;
			self.remove_ui();
			self.clean_up_stage();
			
			/*
			self.character_left.x = self.start_positions[0].x;
			self.character_left.y = self.start_positions[0].y;
			self.character_right.x = self.start_positions[1].x;
			self.character_right.y = self.start_positions[1].y;
			*/
			
			self.character_left.x = self.character_left_positions.start.x;
			self.character_left.y = self.character_left_positions.start.y;
			
			self.character_right.x = self.character_right_positions.start.x;
			self.character_right.y = self.character_right_positions.start.y;
			
			self.holder.setChildIndex(self.character_left, self.holder.getNumChildren()-1);
			self.holder.setChildIndex(self.character_right, self.holder.getNumChildren()-1);
			
			TweenMax.to(self.character_left, 0.15, {ease:Linear.easeOut, x:self.character_left_positions.end.x, y:self.character_left_positions.end.y});
			TweenMax.to(self.character_right, 0.15, {ease:Linear.easeOut, x:self.character_right_positions.end.x, y:self.character_right_positions.end.y});
			
			//TweenMax.to(self.character_left, 0.15, {ease:Linear.easeOut, x:self.end_positions[0].x, y:self.end_positions[0].y});
			//TweenMax.to(self.character_right, 0.15, {ease:Linear.easeOut, x:self.end_positions[1].x, y:self.end_positions[1].y});
			
			var game_over_text = new createjs.Container();
			game_over_text.x = window.gameWidth/2;
			game_over_text.y = 188;
			self.holder.addChild(game_over_text);
			
			var graphic, image_data;
			
			graphic = window.game.sprites.get_bitmap("default_assets", "text_game_over");
			image_data = window.game.sprites.get_data("default_assets", "text_game_over");
			graphic.x = 0 - (image_data.width/2);
			graphic.y = 0 - (image_data.height/2);
			game_over_text.addChild(graphic);
			
			TweenMax.from(game_over_text, 0.25, {delay:0.25, scaleX:0, scaleY:0, ease:Back.easeOut});
			
			var play_again_text = new createjs.Container();
			play_again_text.x = (window.gameWidth/2) + 3;
			play_again_text.y = 365;
			self.holder.addChild(play_again_text);
			
			graphic = window.game.sprites.get_bitmap("default_assets", "text_play_again");
			image_data = window.game.sprites.get_data("default_assets", "text_play_again");
			graphic.x = 0 - (image_data.width/2);
			graphic.y = 0 - (image_data.height/2);
			play_again_text.addChild(graphic);
			
			TweenMax.from(play_again_text, 0.25, {delay:0.25, scaleX:0, scaleY:0, ease:Back.easeOut});
			
			var play_btn = new createjs.Container();
			play_btn.x = 472;
			play_btn.y = 460;
			self.holder.addChild(play_btn);
			
			graphic = window.game.sprites.get_bitmap("default_assets", "btn_play");
			image_data = window.game.sprites.get_data("default_assets", "btn_play");
			graphic.x = 0 - (image_data.width/2);
			graphic.y = 0 - (image_data.height/2);
			play_btn.addChild(graphic);	
			
			TweenMax.from(play_btn, 0.25, {delay:0.45, scaleY:0, ease:Back.easeOut});
			
			var rect = {x:play_btn.x - (image_data.width/2), y:play_btn.y - (image_data.height/2), w:image_data.width, h:image_data.height};
			window.game.mouse.add_click('game_playagain_btn_click', rect, function(){
				window.game.mouse.remove_click("game_playagain_btn_click");
				
				TweenMax.to(game_over_text, 0.35, {scaleX:0, scaleY:0, ease:Linear.easeIn, onComplete:function(){
					self.holder.removeChild(game_over_text);
				}});
				
				TweenMax.to(play_again_text, 0.35, {alpha:0, onComplete:function(){
					self.holder.removeChild(play_again_text);
				}});
				
				TweenMax.to(play_btn, 0.35, {overwrite:true, scaleY:0, ease:Back.easeIn, onComplete:function(){
					self.holder.removeChild(play_btn);
					self.build_page(true);
				}});	
			}, null);
		},
		
		level_complete : function ()
		{
			var self = this;
			
			window.game.sound_managers.sorting_game.play("level_complete");
			
			self.remove_ui();
			self.pause_clock();
			TweenMax.to(self.clock, 0.15, {scaleX:0, scaleY:0, ease:Linear.easeNone, onComplete:function(){
				self.clock.visible = false;
			}});
			
			/*
			self.character_left.x = self.start_positions[0].x;
			self.character_left.y = self.start_positions[0].y;
			self.character_right.x = self.start_positions[1].x;
			self.character_right.y = self.start_positions[1].y;
			
			self.holder.setChildIndex(self.character_left, self.holder.getNumChildren()-1);
			self.holder.setChildIndex(self.character_right, self.holder.getNumChildren()-1);
			
			TweenMax.to(self.character_left, 0.15, {ease:Linear.easeOut, x:self.end_positions[0].x, y:self.end_positions[0].y});
			TweenMax.to(self.character_right, 0.15, {ease:Linear.easeOut, x:self.end_positions[1].x, y:self.end_positions[1].y});
			*/
			
			self.character_left.x = self.character_left_positions.start.x;
			self.character_left.y = self.character_left_positions.start.y;
			
			self.character_right.x = self.character_right_positions.start.x;
			self.character_right.y = self.character_right_positions.start.y;
			
			self.holder.setChildIndex(self.character_left, self.holder.getNumChildren()-1);
			self.holder.setChildIndex(self.character_right, self.holder.getNumChildren()-1);
			
			TweenMax.to(self.character_left, 0.15, {ease:Linear.easeOut, x:self.character_left_positions.end.x, y:self.character_left_positions.end.y});
			TweenMax.to(self.character_right, 0.15, {ease:Linear.easeOut, x:self.character_right_positions.end.x, y:self.character_right_positions.end.y});
			
			
			self.well_done_text = new createjs.Container();
			self.well_done_text.x = window.gameWidth/2;
			self.well_done_text.y = 167;
			self.holder.addChild(self.well_done_text);
			
			var graphic, image_data;
			
			graphic = window.game.sprites.get_bitmap("default_assets", "text_well_done");
			image_data = window.game.sprites.get_data("default_assets", "text_well_done");
			graphic.x = 0 - (image_data.width/2);
			graphic.y = 0 - (image_data.height/2);
			self.well_done_text.addChild(graphic);
			
			TweenMax.from(self.well_done_text, 0.25, {delay:0.25, scaleX:0, scaleY:0, ease:Back.easeOut});
			
			var level_text = new createjs.Container();
			level_text.x = window.gameWidth/2;
			level_text.y = 299;
			self.holder.addChild(level_text);
			
			var graphic, image_data;
			
			graphic = window.game.sprites.get_bitmap("default_assets", "text_level_"+window.game.current_level+"_complete");
			image_data = window.game.sprites.get_data("default_assets", "text_level_"+window.game.current_level+"_complete");
			graphic.x = 0 - (image_data.width/2);
			graphic.y = 0 - (image_data.height/2);
			level_text.addChild(graphic);
			
			TweenMax.from(level_text, 0.4, {delay:0.45, scaleX:0, scaleY:0, ease:Back.easeOut});
			
			var remove_delay = 1.5;
			
			TweenMax.to(level_text, 0.5, {delay:remove_delay, alpha:0, onComplete:function(){
				self.holder.removeChild(level_text);
				
				self.go_to_next_level();
			}});
		},
		
		go_to_next_level: function()
		{
			var self = this;
			self.pause_active = true;
			self.clean_up_stage();
			
			//Move on or end game
			if (window.game.current_level < 5)
			{
				window.game.current_level ++;
				
				TweenMax.to(self.well_done_text, 0.35, {scaleX:0, scaleY:0, ease:Linear.easeIn, onComplete:function(){
					self.holder.removeChild(self.well_done_text);
				}});
					
				self.build_page(true);
			} else {
				self.game_complete();
			}
		},
		
		game_complete:function()
		{
			var self = this;
			
			dax_event('hidden','','&c5_gm_action=completed');
			console.log('event: completed');
			
			var graphic, image_data;
			
			window.game.sound_managers.woo.play("sound");
			
			var game_complete = new createjs.Container();
			game_complete.x = window.gameWidth/2;
			game_complete.y = 258;
			self.holder.addChild(game_complete);
			
			graphic = window.game.sprites.get_bitmap("default_assets", "text_game_complete");
			image_data = window.game.sprites.get_data("default_assets", "text_game_complete");
			graphic.x = 0 - (image_data.width/2);
			graphic.y = 0 - (image_data.height/2);
			game_complete.addChild(graphic);
			
			TweenMax.from(game_complete, 0.25, {delay:0.25, scaleX:0, scaleY:0, ease:Back.easeOut});
			
			var play_again_text = new createjs.Container();
			play_again_text.x = (window.gameWidth/2) + 3;
			play_again_text.y = 365;
			self.holder.addChild(play_again_text);
			
			graphic = window.game.sprites.get_bitmap("default_assets", "text_play_again");
			image_data = window.game.sprites.get_data("default_assets", "text_play_again");
			graphic.x = 0 - (image_data.width/2);
			graphic.y = 0 - (image_data.height/2);
			play_again_text.addChild(graphic);
			
			TweenMax.from(play_again_text, 0.25, {delay:0.25, scaleX:0, scaleY:0, ease:Back.easeOut});
			
			var play_btn = new createjs.Container();
			play_btn.x = 472;
			play_btn.y = 460;
			self.holder.addChild(play_btn);
			
			graphic = window.game.sprites.get_bitmap("default_assets", "btn_play");
			image_data = window.game.sprites.get_data("default_assets", "btn_play");
			graphic.x = 0 - (image_data.width/2);
			graphic.y = 0 - (image_data.height/2);
			play_btn.addChild(graphic);	
			
			TweenMax.from(play_btn, 0.25, {delay:0.45, scaleY:0, ease:Back.easeOut});
			
			var rect = {x:play_btn.x - (image_data.width/2), y:play_btn.y - (image_data.height/2), w:image_data.width, h:image_data.height};
			window.game.mouse.add_click('game_play_btn_click', rect, function(){
				window.game.mouse.remove_click("game_play_btn_click");
				
				dax_event('hidden','','&c5_gm_action=started');
				console.log('event: start');
				
				TweenMax.to(self.well_done_text, 0.35, {scaleX:0, scaleY:0, ease:Linear.easeIn, onComplete:function(){
					self.holder.removeChild(self.well_done_text);
				}});
				
				TweenMax.to(game_complete, 0.35, {alpha:0, onComplete:function(){
					self.holder.removeChild(game_complete);
				}});
				
				TweenMax.to(play_again_text, 0.35, {alpha:0, onComplete:function(){
					self.holder.removeChild(play_again_text);
				}});
				
				TweenMax.to(play_btn, 0.35, {overwrite:true, scaleY:0, ease:Back.easeIn, onComplete:function(){
					window.game.current_level = 1;
					self.holder.removeChild(play_btn);
					self.build_page(true);
				}});	
			}, null);
		},
		
		clean_up_stage : function()
		{
			var self = this;
			
			for (var i=0; i < self.bucket_array.length; i++)
			{
				self.holder.removeChild(self.bucket_array[i].inside);
				self.holder.removeChild(self.bucket_array[i].container);
			}
			
			TweenMax.killTweensOf(self.holder);
			clearInterval(self.spark_timer);
			
			for (var i=0; i < self.dropped_items_array.length; i++)
			{
				self.holder.removeChild(self.dropped_items_array[i]);
			}
			
			self.dropped_items_array = [];
			
			self.clock.visible = false;
		},
		
		update : function ()
		{
			var self = this;
			self.canvas.update();
			
			if (self.selected_item_data.name != "")
			{
				if (window.isMobile == true)
				{
					self.selected_item.x = window.game.mouse.mouse_position.x; 
					self.selected_item.y = window.game.mouse.mouse_position.y-50; 	
				} else {
					self.selected_item.x = window.game.mouse.mouse_position.x; 
					self.selected_item.y = window.game.mouse.mouse_position.y; 	
				}
				
				self.check_position();
			}
		}
	});
	
});