define([
         "util/classutils",
         "preloadjs",
         ],
         function(classUtils, preloadjs) {
	
	"use strict";
	
	return classUtils.create({
		canvas:null,
		holder:null,
		
		pointer:null,
		trumpet:null,
		play_btn:null,
		arrow:null,
		
		play_btn_scale:0,
		
		pointer_array:[{x:240, y:446}, {x:419, y:460}, {x:400, y:222}, {x:550, y:210}],
		trumpet_array:[{x:457, y:439}, {x:380, y:194}, {x:380, y:240}, {x:539, y:178}],
		
		forced:false,
		
		positive_face:null,
		negative_face:null,
				
		init: function()
		{
			var self = this;
						
			self.canvas = new createjs.Stage("game_canvas");
						
			self.holder = new createjs.Container();
			self.canvas.addChild(self.holder);
			
			var background = window.game.sprites.get_bitmap("instructions_background");
			self.holder.addChild(background);
			
			var lozenge = new createjs.Container();
			lozenge.x = window.gameWidth/2;
			lozenge.y = window.gameHeight/2;
			self.holder.addChild(lozenge);
			
			var graphic, image_data;
			
			graphic = window.game.sprites.get_bitmap("instructions_lozenge");
			image_data = window.game.sprites.get_data("instructions_lozenge");
			graphic.x = 0 - (image_data.width/2);
			graphic.y = 0 - (image_data.height/2);
			lozenge.addChild(graphic);
			
			var buckets = new createjs.Container();
			buckets.x = 471;
			buckets.y = 280;
			self.holder.addChild(buckets);
			
			graphic = window.game.sprites.get_bitmap("instructions_bucket");
			image_data = window.game.sprites.get_data("instructions_bucket");
			graphic.x = 0 - (image_data.width/2);
			graphic.y = 0 - (image_data.height/2);
			buckets.addChild(graphic);
			
			var bucket_obj_2 = window.game.sprites.get_bitmap("game_object_2");
			bucket_obj_2.scaleX = bucket_obj_2.scaleY = .8;
			bucket_obj_2.x = -143;
			bucket_obj_2.y = -24;
			buckets.addChild(bucket_obj_2);
			
			var bucket_obj_1 = window.game.sprites.get_bitmap("game_object_1");
			bucket_obj_1.scaleX = bucket_obj_1.scaleY = .8;
			bucket_obj_1.x = 47;
			bucket_obj_1.y = -24;
			buckets.addChild(bucket_obj_1);
			
			self.play_btn = new createjs.Container();
			self.play_btn.x = 740;
			self.play_btn.y = 435;
			self.play_btn.scaleX = self.play_btn.scaleY = self.play_btn_scale = 0.65;
			self.holder.addChild(self.play_btn);
			
			graphic = window.game.sprites.get_bitmap("default_assets", "btn_play");
			image_data = window.game.sprites.get_data("default_assets", "btn_play");
			graphic.x = 0 - (image_data.width/2);
			graphic.y = 0 - (image_data.height/2);
			self.play_btn.addChild(graphic);
			
			self.arrow = new createjs.Container();
			self.arrow.x = 293;
			self.arrow.y = 395;
			self.arrow.alpha = 0;
			self.holder.addChild(self.arrow);
			
			graphic = window.game.sprites.get_bitmap("default_assets", "instructions_arrow");
			image_data = window.game.sprites.get_data("default_assets", "instructions_arrow");
			graphic.x = 0 - (image_data.width/2);
			graphic.y = 0 - (image_data.height/2);
			self.arrow.addChild(graphic);
			
			self.trumpet = new createjs.Container();
			self.trumpet.x = 457;
			self.trumpet.y = 439;
			self.holder.addChild(self.trumpet);
			
			graphic = window.game.sprites.get_bitmap("game_object_1");
			image_data = window.game.sprites.get_data("game_object_1");
			graphic.x = 0 - (image_data.width/2);
			graphic.y = 0 - (image_data.height/2);
			self.trumpet.addChild(graphic);
			
			self.pointer = new createjs.Container();
			self.pointer.x = 240;
			self.pointer.y = 446;
			self.holder.addChild(self.pointer);
			
			var pointer_type = "hand";
			
			if (!window.isMobile)
				pointer_type = "cursor";
			
			graphic = window.game.sprites.get_bitmap("default_assets", pointer_type);
			image_data = window.game.sprites.get_data("default_assets", pointer_type);
			graphic.x = 0 - (image_data.width/2);
			graphic.y = 0 - (image_data.height/2);
			self.pointer.addChild(graphic);
		},
		
		show : function (forced)
		{
			var self = this;	
			self.holder.visible = true;
			TweenMax.to(self.holder, 0.5, {alpha:1});
			
			self.forced = forced;
			
			self.build_page();
		},
		
		show_from_game : function()
		{
			var self = this;
			self.holder.visible = true;
			TweenMax.to(self.holder, 0.5, {alpha:1});
			
			self.build_page();
			self.play_btn.visible = false;
			window.game.mouse.remove_click("instructions_play_btn_click");
			
			var close_btn = new createjs.Container();
			close_btn.x = 816;
			close_btn.y = 79;
			self.holder.addChild(close_btn);
			
			var graphic, image_data;
			graphic = window.game.sprites.get_bitmap("default_assets", "btn_close");
			image_data = window.game.sprites.get_data("default_assets", "btn_close");
			graphic.x = 0 - (image_data.width/2);
			graphic.y = 0 - (image_data.height/2);
			close_btn.addChild(graphic);
			
			var rect = {x:close_btn.x - (image_data.width/2), y:close_btn.y - (image_data.height/2), w:image_data.width, h:image_data.height};
			window.game.mouse.add_click('instructions_close_click', rect, function(){
				window.game.mouse.remove_click("instructions_close_click");
				TweenMax.to(close_btn, 0.35, {overwrite:true, scaleY:0, ease:Back.easeIn, onComplete:function(){
					self.kill_instructions();
					window.game.game_instructions_hide();
					self.holder.removeChild(close_btn);
				}});	
				window.game.sound_managers.general.play("pop");	
			}, null);
		},
		
		hide : function ()
		{
			var self = this;
			
			TweenMax.to(self.holder, 0.35, {alpha:0, onComplete:function(){
				self.holder.visible = false;
			}});
		},
		
		build_page: function()
		{
			var self = this;
			
			self.trumpet.x = self.trumpet_array[0].x;
			self.trumpet.y = self.trumpet_array[0].y;
			self.pointer.x = self.pointer_array[0].x;
			self.pointer.y = self.pointer_array[0].y;
			self.arrow.alpha = 0;
			self.play_btn.scaleY = 0.65;
			
			self.step_1();
			
			var image_data = window.game.sprites.get_data("default_assets", "btn_play");
			
			var rect = {x:self.play_btn.x - (image_data.width/2), y:self.play_btn.y - (image_data.height/2), w:image_data.width, h:image_data.height};
			window.game.mouse.add_click('instructions_play_btn_click', rect, function(){
				window.game.mouse.remove_click("instructions_play_btn_click");
				TweenMax.to(self.play_btn, 0.35, {overwrite:true, scaleY:0, ease:Back.easeIn, onComplete:function(){
					window.game.updateState("game");
					self.kill_instructions();
				}});	
				window.game.sound_managers.general.play("pop");	
			}, null);
		},
		
		step_1 : function()
		{
			var self = this;
			TweenMax.to(self.pointer, 0.5, {delay:0.5, x:self.pointer_array[1].x, y:self.pointer_array[1].y, onComplete:function(){
				self.step_2();
			}});	
		},
		
		step_2 : function()
		{
			var self = this;
			TweenMax.to(self.pointer, 1, {bezier:[{x:self.pointer_array[2].x - 100, y:self.pointer_array[2].y + (self.pointer_array[2].y/2)}, {x:self.pointer_array[2].x, y:self.pointer_array[2].y}]});	
			TweenMax.to(self.trumpet, 1, {bezier:[{x:self.trumpet_array[1].x - 100, y:self.trumpet_array[1].y + (self.trumpet_array[1].y/2)}, {x:self.trumpet_array[1].x, y:self.trumpet_array[1].y}], onComplete:function(){
				self.step_3();
			}});
		},
		
		step_3 : function()
		{
			var self = this;
			TweenMax.to(self.trumpet, 0.25, {x:self.trumpet_array[2].x, y:self.trumpet_array[2].y, alpha:0, onComplete:function(){
				window.game.sound_managers.sorting_game.play("incorrect");	
				self.step_4();
			}});
			TweenMax.to(self.pointer, 1, {bezier:[{x:self.pointer_array[2].x - 100, y:self.pointer_array[2].y + (self.pointer_array[2].y/2)}, {x:self.pointer_array[0].x, y:self.pointer_array[0].y}]});	
		},
		
		step_4 : function ()
		{
			var self = this;
			self.negative_face = new createjs.Container();
			self.negative_face.x = 472;
			self.negative_face.y = 288;
			self.holder.addChild(self.negative_face);
			
			var graphic = window.game.sprites.get_bitmap("default_assets", "instructions_sad");
			var image_data = window.game.sprites.get_data("default_assets", "instructions_sad");
			graphic.x = 0 - (image_data.width/2);
			graphic.y = 0 - (image_data.height/2);
			self.negative_face.addChild(graphic);	
			
			TweenMax.from(self.negative_face, 0.25, {scaleX:0, scaleY:0, ease:Back.easeOut, onComplete:function(){
				TweenMax.to(self.negative_face, 0.35, {delay:0.85, alpha:0, onComplete:function(){
					self.holder.removeChild(self.negative_face);
					self.step_5();
				}});
			}});
			
			TweenMax.to(self.trumpet, 0.25, {delay:0.25, x:self.trumpet_array[1].x, y:self.trumpet_array[1].y, alpha:1, onComplete:function(){
				TweenMax.to(self.arrow, 0.15, {alpha:1});
				TweenMax.to(self.trumpet, 1, {bezier:[{x:self.trumpet_array[1].x - 100, y:self.trumpet_array[1].y + (self.trumpet_array[1].y/2)}, {x:self.trumpet_array[0].x, y:self.trumpet_array[0].y}]});
			}});
		},
		
		step_5 : function()
		{
			var self = this;
			TweenMax.to(self.arrow, 0.15, {alpha:0});
			
			TweenMax.to(self.pointer, 0.5, {delay:0.5, x:self.pointer_array[1].x, y:self.pointer_array[1].y, onComplete:function(){
				self.step_6();
			}});
		},
		
		step_6 : function()
		{
			var self = this;
			TweenMax.to(self.pointer, 1, {bezier:[{x:self.pointer_array[3].x + 100, y:self.pointer_array[3].y + (self.pointer_array[3].y/2)}, {x:self.pointer_array[3].x, y:self.pointer_array[3].y}]});	
			TweenMax.to(self.trumpet, 1, {bezier:[{x:self.trumpet_array[3].x + 100, y:self.trumpet_array[3].y + (self.trumpet_array[3].y/2)}, {x:self.trumpet_array[3].x, y:self.trumpet_array[3].y}], onComplete:function(){
				self.step_7();
			}});	
		},
		
		step_7 : function()
		{
			var self = this;
			TweenMax.to(self.trumpet, 0.25, {x:self.trumpet_array[3].x, y:self.trumpet_array[3].y, alpha:0, onComplete:function(){
				window.game.sound_managers.sorting_game.play("correct");	
				self.step_8();
			}});	
			TweenMax.to(self.pointer, 1, {bezier:[{x:self.pointer_array[3].x + 100, y:self.pointer_array[3].y + (self.pointer_array[3].y/2)}, {x:self.pointer_array[0].x, y:self.pointer_array[0].y}]});
		},
		
		step_8 : function()
		{
			var self = this;
			
			self.positive_face = new createjs.Container();
			self.positive_face.x = 472;
			self.positive_face.y = 288;
			self.holder.addChild(self.positive_face);
			
			var graphic = window.game.sprites.get_bitmap("default_assets", "instructions_happy");
			var image_data = window.game.sprites.get_data("default_assets", "instructions_happy");
			graphic.x = 0 - (image_data.width/2);
			graphic.y = 0 - (image_data.height/2);
			self.positive_face.addChild(graphic);	
			
			TweenMax.from(self.positive_face, 0.25, {scaleX:0, scaleY:0, ease:Back.easeOut, onComplete:function(){
				TweenMax.to(self.positive_face, 0.35, {delay:0.65, alpha:0, onComplete:function(){
					self.holder.removeChild(self.positive_face);
				}});
			}});
			
			self.trumpet.x = self.trumpet_array[0].x;
			self.trumpet.y = self.trumpet_array[0].y;
			
			TweenMax.to(self.trumpet, 0.25, {delay:0.85, alpha:1, onComplete:function(){
				self.step_1();
			}});
		},
		
		kill_instructions : function()
		{
			var self = this;
			TweenMax.killTweensOf(self.trumpet);
			TweenMax.killTweensOf(self.pointer);
			
			TweenMax.killTweensOf(self.negative_face);
			TweenMax.killTweensOf(self.positive_face);
			
			self.holder.removeChild(self.positive_face);
			self.holder.removeChild(self.negative_face);	
		},
		
		update : function ()
		{
			var self = this;
			self.canvas.update();
		}
	});
	
});