define([
         "util/classutils",
         "easeljs",
         ],
         function(classUtils, easelJS) {
	
	"use strict";
	
	return classUtils.create({
		
		canvas:null,
		key_pressed:0,
		
		menu_loop:[],
		world_loop:[],
		
		menu_menu_highlight:null,
		world_highlight:null,
		world_highlight_width: 172,
		world_highlight_height: 110,
		
		previous_state:'',
		current_id:0,
		current_position:null,
		
		keyboard_active:false,
		
		mode:'menu',
		
		init:function(_canvas)
		{
			var self = this;
			self.canvas = _canvas;
			
			self.menu_highlight = new createjs.Shape();
			self.menu_highlight.alpha = 0.5;
			self.menu_highlight.scaleX = self.menu_highlight.scaleY = 0;
			self.canvas.addChild(self.menu_highlight);
			self.menu_highlight.visible = false;
			
			self.world_highlight = new createjs.Shape();
			self.world_highlight.scaleX = self.world_highlight.scaleY = 0;
			self.canvas.addChild(self.world_highlight);
			self.world_highlight.visible = false;
			
			if (!window.ada_is_mobile)
			{				
				//To stop tab changing focus this must be a keydown
				window.addEventListener('keydown',function(e){
					self.keyboard_event(e);
				},false);
			}
		},
		
		keyboard_event : function (e)
		{
			var self = this;
			self.key_pressed = e.keyCode ? e.keyCode : e.which;
			var active_keys = {left:37, right:39, up:38, down:40, tab:9, enter:13, space:32};
			
			if (self.keyboard_active == false)
			{
				for (var i in active_keys)
				{
					if (self.key_pressed == active_keys[i])
					{
						self.keyboard_active = true;
						self.show_highlight(self.current_position);
					}
				}	
			}
			
			switch(self.key_pressed)
			{
				case active_keys.left:
					e.preventDefault();
					if (self.mode == "menu")
						self.update_menu_position('left');
					else
						self.update_world_position('left');
				break;
				case active_keys.right:
					e.preventDefault();
					if (self.mode == "menu")
						self.update_menu_position('right');
					else
						self.update_world_position('right');
				break;
				case active_keys.up:
					e.preventDefault();
					if (self.mode == "menu")
						self.update_menu_position('up');
					else
						self.update_world_position('up');
				break;
				case active_keys.down:
					e.preventDefault();
					if (self.mode == "menu")
						self.update_menu_position('down');
					else
						self.update_world_position('down');
				break;
				case active_keys.tab:
					e.preventDefault();
					self.tab_through();
				break;
				case active_keys.enter:
				case active_keys.space:
					e.preventDefault();
					self.make_callback();
				break;
			}
		},
		
		add_to_menu_loop : function(id, position, callback, params)
		{
			var self = this;
			var element = {id:id, x:position.x, y:position.y, width:position.w, height:position.h, callback:callback, params:params};
			if (self.menu_loop.length == 0)
			{
				self.current_position = element;
			}
			self.menu_loop.push(element);	
		},
		
		init_world_keyboard_control : function ()
		{
			var self = this;
			
			var total_areas = 23; //including the bbc gui and our gui
			var bbc_gui_size = 56;
			var current_x_point = 15;
			var current_y_point = 80;
			var horz_gap = 10;
			var vert_gap = 10;
			
			for (var i = 0; i < total_areas; i++)
			{
				var element;
				if (i == 0)
				{
					element = {id:'pause_btn', x:window.ada.hud.pause_btn.x, y:window.ada.hud.pause_btn.y, w:bbc_gui_size, h:bbc_gui_size};
				} else if (i == 1)
				{
					element = {id:'help_btn', x:window.ada.hud.help_btn.x, y:window.ada.hud.help_btn.y, w:bbc_gui_size, h:bbc_gui_size};
				} else if (i == 2)
				{
					element = {id:'audio_btn', x:window.ada.hud.audio_on_btn.x, y:window.ada.hud.audio_on_btn.y, w:bbc_gui_size, h:bbc_gui_size};
				} else {
					element = {id:'world_area_' + i, x:current_x_point + horz_gap, y:current_y_point + vert_gap, w:self.world_highlight_width, h:self.world_highlight_height};
					
					current_x_point = current_x_point + horz_gap + self.world_highlight_width;
					
					if (current_x_point + self.world_highlight_width >= window.ada_width)
					{
						current_x_point = 15;
						current_y_point = current_y_point + vert_gap + self.world_highlight_height;
					}
				}
				self.world_loop.push(element);
			}
			
			/*
			for (var j = 0; j < self.world_loop.length; j++)
			{
				var piece_data = self.world_loop[j];
				
				var container = new createjs.Shape();
				container.x = piece_data.x + (piece_data.w/2);
				container.y = piece_data.y + (piece_data.h/2);
				container.scaleX = container.scaleY = 0;
				container.alpha = 0.5;
				self.canvas.addChild(container);
				
				var graphic = container.graphics;
				graphic.beginFill("#ffffff");
				graphic.drawRect(0 - (piece_data.w/2), 0 - (piece_data.h/2), piece_data.w, piece_data.h);
				graphic.endFill();
				
				TweenMax.to(container, 0.25, {scaleX:1, scaleY:1, ease:Bounce.easeOut, delay:0.05 * j});
			}
			*/
			
			self.current_position = self.world_loop[2];
		},
		
		show_highlight : function(element)
		{
			var self = this;	
			
			if (element)
			{
				if (self.mode == "menu")
				{
					self.menu_highlight.visible = true;
				
					var graphics = self.menu_highlight.graphics;
					graphics.clear();
					graphics.beginFill("#0077ca");
					graphics.drawEllipse(0 - (element.width/2), 0 - (element.height/2), element.width, element.height);
					graphics.endFill();	
					
					TweenMax.killTweensOf(self.menu_highlight);
					
					self.menu_highlight.x = element.x + (element.width/2);
					self.menu_highlight.y = element.y + (element.height/2);
					self.menu_highlight.scaleX = self.menu_highlight.scaleY = 0;
					self.menu_highlight.alpha = 0.5;
					
					TweenMax.to(self.menu_highlight, 0.35, {scaleX:1, scaleY:1, ease:Back.easeOut});			
				} else {
					self.world_highlight.visible = true;
					
					var graphics = self.world_highlight.graphics;
					graphics.clear();
					graphics.setStrokeStyle(3,"square");
					graphics.beginStroke("#ffffff");
					graphics.beginFill("rgba(0,119,202,0.25)");
					graphics.drawRect(0 - (element.w/2), 0 - (element.h/2), element.w, element.h);
					graphics.endStroke();
					graphics.endFill();
					
					TweenMax.killTweensOf(self.world_highlight);
					self.world_highlight.x = element.x + (element.w/2);
					self.world_highlight.y = element.y + (element.h/2);
					self.world_highlight.scaleX = self.world_highlight.scaleY = 0;
					
					TweenMax.to(self.world_highlight, 0.35, {scaleX:1, scaleY:1});
				}
				
			}
		},
		
		hide_highlight : function()
		{
			var self = this;
			self.menu_highlight.visible = false;	
			self.world_highlight.visible = false;
		},
		
		tab_through : function ()
		{
			var self = this;
			
			var new_element = self.current_position;
			if (self.mode == "menu")
			{
				for (var i=0; i<self.menu_loop.length;i++)
				{
					if (self.current_position == self.menu_loop[i])
					{
						if (self.menu_loop[i+1])
						{
							new_element = self.menu_loop[i+1];	
						} else {
							new_element = self.menu_loop[0];
						}
					}
				}
			} else {
				for (var j=0; j<self.world_loop.length; j++)
				{
					if (self.current_position == self.world_loop[j])
					{
						if (self.world_loop[j+1])
						{
							new_element = self.world_loop[j+1];
						} else {
							new_element = self.world_loop[0];
						}
					}
				}
			}
			
			self.current_position = new_element;
			self.show_highlight(new_element);
		},
		
		update_world_position : function (direction)
		{
			var self = this;
			
			if (self.current_position == null)
			{
				self.current_position = self.world_loop[3];
			}
			
			var new_element = self.current_position;
			
			if (self.world_highlight.visible == false)
			{
				self.show_highlight(self.world_loop[3]);
			}
			
			world_loop: for (var i=0; i<self.world_loop.length;i++)
			{
				if (self.world_loop[i] == self.current_position)
				{
					if (direction == 'left')
					{
						if (i - 1 != 2)
						{
							if (i == 0)
							{
								new_element = self.world_loop[3];
							} else {
								if (self.world_loop[i - 1])
								{	
									new_element = self.world_loop[i-1];
									break world_loop;
								}		
							}
						}
					} else if (direction == 'right')
					{
						if (self.world_loop[i+1])
						{
							new_element = self.world_loop[i+1];
							break world_loop;
						}
					} else if (direction == 'up')
					{
						if (i != 0 && i != 1 && i != 2)
						{
							if (self.world_loop[i-5])
							{
								new_element = self.world_loop[i-5];
								break world_loop;
							} else {
								new_element = self.world_loop[0];
								break world_loop;
							}	
						}
					} else if (direction == 'down')
					{
						if (self.world_loop[i+5])
						{
							new_element = self.world_loop[i+5];
							break world_loop;
						}
					}	
				}
			}
			self.current_position = new_element;
			self.show_highlight(new_element);
		},		
		
		update_menu_position : function (direction)
		{
			var self = this;
			var new_element = self.current_position;
			var i = 0;
			if (direction == 'left')
			{
				direction_left_loop:
				for (i = 0; i<self.menu_loop.length; i++)
				{
					if (self.menu_loop[i].x < self.current_position.x)
					{
						if (self.menu_loop[i].x < new_element.x)
						{
							var in_line_with_element = self.check_if_element_is_inline_and_closest(self.menu_loop[i], direction);
							
							if (!in_line_with_element)
							{
								var is_closest_element = self.check_is_closest(self.menu_loop[i], direction, self.menu_loop);
							
								if (is_closest_element == true)
								{
									new_element = self.menu_loop[i];	
									break direction_left_loop;
								}	
							} else {
								
								new_element = in_line_with_element;
								break direction_left_loop;	 
							}
						}
					}
				}
			} else if (direction == "right")
			{
				direction_right_loop:
				for (i = 0; i<self.menu_loop.length; i++)
				{
					if (self.menu_loop[i].x > self.current_position.x)
					{
						if (self.menu_loop[i].x > new_element.x)
						{
							var in_line_with_element = self.check_if_element_is_inline_and_closest(self.menu_loop[i], direction);
							
							if (!in_line_with_element)
							{
								var is_closest_element = self.check_is_closest(self.menu_loop[i], direction, self.menu_loop);
							
								if (is_closest_element == true)
								{
									new_element = self.menu_loop[i];	
									break direction_right_loop;
								}
							} else {
								new_element = in_line_with_element;	
								break direction_right_loop;
							}
						}
					}
				}
			} else if (direction == "up")
			{
				direction_up_loop:
				for (i = 0; i<self.menu_loop.length; i++)
				{
					if (self.menu_loop[i].y < self.current_position.y)
					{
						if (self.menu_loop[i].y < new_element.y)
						{	
							var in_line_with_element = self.check_if_element_is_inline_and_closest(self.menu_loop[i], direction);
							
							if (!in_line_with_element)
							{
								var is_closest_element = self.check_is_closest(self.menu_loop[i], direction, self.menu_loop);
							
								if (is_closest_element == true)
								{
									new_element = self.menu_loop[i];	
									break direction_up_loop;
								}
							} else {
								new_element = in_line_with_element;	
								break direction_up_loop;
							}
						}
					}
				}
			} else if (direction == "down")
			{
				direction_down_loop:
				for (i = 0; i<self.menu_loop.length; i++)
				{
					if (self.menu_loop[i].y > self.current_position.y)
					{
						if (self.menu_loop[i].y > new_element.y)
						{	
							var in_line_with_element = self.check_if_element_is_inline_and_closest(self.menu_loop[i], direction);
							
							if (!in_line_with_element)
							{
								var is_closest_element = self.check_is_closest(self.menu_loop[i], direction, self.menu_loop);
							
								if (is_closest_element == true)
								{
									new_element = self.menu_loop[i];	
									break direction_down_loop;
								}
							} else {
								new_element = in_line_with_element;
								break direction_down_loop;	
							}
						}
					}
				}
			}
			
			self.current_position = new_element;
			
			self.show_highlight(new_element);
		},
		
		check_if_element_is_inline_and_closest : function(proposed_element, direction)
		{
			var self = this;
			var tolerance = 20		;
			var on_line_and_closest_element = null;
			var on_line_array = [];
			
			for (var i=0; i<self.menu_loop.length; i++)
			{
				if (self.menu_loop[i] != self.current_position)
				{
					if (direction == "left" || direction == "right")
					{
						if (self.menu_loop[i].y >= (self.current_position.y - tolerance) && self.menu_loop[i].y <= (self.current_position.y + tolerance))
						{
							on_line_array.push(self.menu_loop[i]);
						}	
					} else if (direction == "up" || direction == "down")
					{
						if (self.menu_loop[i].x >= (self.current_position.x - tolerance) && self.menu_loop[i].x <= (self.current_position.x + tolerance))
						{
							on_line_array.push(self.menu_loop[i]);
						}
					}	
				}
			}
			
			if (on_line_array.length == 0)
			{
				on_line_and_closest_element = null;
			}
			
			on_line_loop:
			for (var k=0; k<on_line_array.length;k++)
			{
				var element = on_line_array[k];
				var closest_element = false;
				
				closest_element = self.check_is_closest(element, direction, on_line_array);
				
				if (closest_element == true)
				{
					on_line_and_closest_element = on_line_array[k];
				
					if (direction == "left")
					{
						if (on_line_and_closest_element.x >= self.current_position.x)
						{
							on_line_and_closest_element = null;
						}
					} else if (direction == "right")
					{
						if (on_line_and_closest_element.x <= self.current_position.x)
						{
							on_line_and_closest_element = null;
						}
					} else if (direction == "down")
					{
						if (on_line_and_closest_element.y <= self.current_position.y)
						{
							on_line_and_closest_element = null;
						}
					} else if (direction == "up")
					{
						if (on_line_and_closest_element.y >= self.current_position.y)
						{
							on_line_and_closest_element = null;
						}
					}
					break on_line_loop;
				}
			}
			
			return on_line_and_closest_element;
		},
		
		check_is_closest : function (proposed_element, direction, array)
		{
			var self = this;
			var closest = true;
			
			var distance;
			if (direction == "left" || direction == "right")
			{
				if (proposed_element.x >= self.current_position.x)
				{
					distance = (proposed_element.x - self.current_position.x) - self.current_position.x;
				} else {
					distance = self.current_position.x - proposed_element.x;
				}
			} else {
				if (proposed_element.y >= self.current_position.y)
				{
					distance = (proposed_element.y - self.current_position.y) - self.current_position.y;	
				} else {
					distance = self.current_position.y - proposed_element.y;
				}
			}

			for (var i = 0; i<array.length; i++)
			{
				if (self.current_position != array[i])
				{
					if (array[i] != proposed_element)
					{
						var new_distance;
						if (direction == "left" || direction == "right")
						{
							if (array[i].x >= self.current_position.x)
							{
								new_distance = (array[i].x - self.current_position.x) - self.current_position.x;
							} else {
								new_distance = self.current_position.x - array[i].x;
							}
						} else {
							if (array[i].y >= self.current_position.y)
							{
								new_distance = (array[i].y - self.current_position.y) - self.current_position.y;	
							} else {
								new_distance = self.current_position.y - array[i].y;
							}
						}
						
						if (new_distance > 0)
						{
							if (new_distance < distance)
							{
								closest = false;
							}	
						}		
					}
				}	
			}	
			
			return closest;
		},
		
		make_callback : function()
		{
			var self = this;
			if (self.mode == "menu")
			{
				var params = self.current_position.params;
				if (params)
				{
					self.current_position.callback.call(null, params);		
				} else {
					self.current_position.callback.call();	
				}	
			} else {
				TweenMax.to(self.world_highlight, 0.35, {scaleX:0, scaleY:0, ease:Linear.easeIn, onComplete:function(){
					TweenMax.to(self.world_highlight, 0.25, {scaleX:1, scaleY:1, delay:0.5, ease:Linear.easeOut});
				}});
				if (self.current_position.id == 'pause_btn')
				{
					window.ada.hud.pause_btn_click();
				} else if (self.current_position.id == "help_btn")
				{
					window.ada.hud.help_btn_click();
				} else if (self.current_position.id == "audio_btn")
				{
					if (window.ada.audio_manager.muted)
					{
						window.ada.hud.audio_off_click();
					} else {
						window.ada.hud.audio_on_click();
					}
				} else if (self.current_position.id == "world_area_22") {
					window.ada.hud.badge_btn_click();
				} else if (self.current_position.id == "world_area_18"){
					window.ada.hud.gizmo_btn_click();
				} else {
					window.ada.scene.scene_action(self.current_position.x + (self.current_position.w/2), self.current_position.y + (self.current_position.h/2));	
				}	
			}
		},
		
		remove_element : function (id)
		{
			var self = this;
			
			for (var i=0; i<self.menu_loop.length;i++)
			{
				if (id == self.menu_loop[i].id)
				{
					var object_deleted = false;
					if (self.current_position == self.menu_loop[i])
					{
						object_deleted = true;
					}

					self.menu_loop.splice(i, 1);
					
					if (object_deleted == true)
					{
						self.current_position = self.menu_loop[0];
					}
				}
			}
		},
		
		reset_manager : function ()
		{
			var self = this;	
			self.menu_loop = [];
			self.menu_highlight.visible = false;
			self.current_position = null;
		},		
		
		update : function()
		{
			var self = this;
			
			if (window.ada.state != self.previous_state)
			{
				self.menu_highlight.visible = false;
				self.world_highlight.visible = false;
			}
			
			self.canvas.removeChild(self.menu_highlight);
			self.canvas.addChild(self.menu_highlight);
			
			self.canvas.removeChild(self.world_highlight);
			self.canvas.addChild(self.world_highlight);
			
			self.previous_state = window.ada.state;
		}
		
	});
});