define([
         "util/classutils",
         ],
         function(classUtils) {
	
	"use strict";
	
	return classUtils.create({
		
		canvas:null,
		clicks:{},
		ups:{},
		overs:{},
		outs:{},
		mouse_position:null,
		click_debug:false,
	
		
		init:function(_canvas)
		{
			var self = this;
			self.canvas = _canvas;
			
			self.canvas.addEventListener("stagemousedown", function(e){
				 self.click_event(e);
			});
			self.canvas.addEventListener("stagemousemove", function(e){
			 	self.move_event(e);
			});
			
			 self.canvas.addEventListener("stagemouseup", function(e){
				 self.stage_up_event(e);
		 	});
		},
		
		add_click:function(id, rect, callback, data)
		{
			var self = this;
			
			if (self.click_debug)
			{
				var rectangle = new createjs.Shape();
				rectangle.alpha = 0.2;
				rectangle.graphics.beginFill("#0006ff").drawRect(rect.x, rect.y, rect.w, rect.h);
			
				self.canvas.addChild(rectangle);
				
/* 				console.log("Drawn rectangle at x = " + rect.x + ", y = " + rect.y + ", width = " + rect.w + ", height = " + rect.h); */
			}
			
			self.add(self.clicks, {id:id, rect:rect, callback:callback, data:data}, 'click');
		},
		
		add_stage_up:function(id, rect, callback, data)
		{
			var self = this;
			 
			self.add(self.ups, {id:id, rect:rect, callback:callback, data:data}, 'stage_up');
			
		},
		
		remove_stage_up:function(id)
		{
			var self = this;
			 
			delete self.ups[id];
		},
	 
		
		remove_click:function(id)
		{
			var self = this;
			delete self.clicks[id];
		},
		
		
		add_over:function(id, rect, callback, data)
		{
			var self = this;
			self.add(self.overs, {id:id, rect:rect, callback:callback, data:data}, 'over');
		},
		
		remove_over:function(id)
		{
			var self = this;
			delete self.overs[id];
		},
		
		add_out:function(id, rect, callback, data)
		{
			var self = this;
			self.add(self.outs, {id:id, rect:rect, callback:callback, data:data}, 'out');
		},
		
		remove_out:function(id)
		{
			var self = this;
			delete self.outs[id];
		},
	
		add:function(obj, item, type)
		{
			var self = this;
			 
			if(obj[item.id] != undefined)
			{
				console.log("WARNING: Cannot add listener (" + type + ") to '" + item.id + "' as a listener already exists for this id. The id must be unique.");
				return null;
			}
			
			obj[item.id] = item;
		},
		
		click_event:function(e)
		{
			var self = this;
			var x = e.stageX;
			var y = e.stageY;
			 
			
			for(var i in self.clicks)
			{	
				if(self.get_collision({x:x, y:y}, self.clicks[i].rect))
				{
					self.clicks[i].callback.apply(null, [self.clicks[i].data]);
				}	
			}	
		},
		
		move_event:function(e)
		{
			var self = this;
			var x = e.stageX;
			var y = e.stageY;
		 
			if(self.mouse_position != null)
			{
				for(var i in self.overs)
				{
					if(self.get_collision({x:x, y:y}, self.overs[i].rect))
					{
						if(!self.get_collision(self.mouse_position, self.overs[i].rect))
						{
							self.overs[i].callback.apply(null, [self.overs[i].data]);
						}	
					}	
				}
				for(var i in self.outs)
				{
					if(!self.get_collision({x:x, y:y}, self.outs[i].rect))
					{
						if(self.get_collision(self.mouse_position, self.outs[i].rect))
						{
							self.outs[i].callback.apply(null, [self.outs[i].data]);
						}	
					}	
				}
			}	
			
			self.mouse_position = {x:x, y:y};
		},
		
		stage_up_event:function(e)
		{
			
		 
			var self = this;
			var x = e.stageX;
			var y = e.stageY;
			
			 
			for(var i in self.ups)
			{	
				if(self.get_collision({x:x, y:y}, self.ups[i].rect))
				{
					
					self.ups[i].callback.apply(null, [self.ups[i].data]);
					 
				}	
			}	
		
		},
		
		get_collision:function(point, rect)
		{
			var self = this;
			
			var w;
			var h;
			
			if(rect.width != undefined && rect.w == undefined)
				w = rect.width;
			else if(rect.w != undefined && rect.width == undefined)
				w = rect.w;
				
			if(rect.height != undefined && rect.h == undefined)
				h = rect.height;
			else if(rect.h != undefined && rect.height == undefined)
				h = rect.h;
			
			if(point.x > rect.x && point.x < rect.x + w &&
			   point.y > rect.y && point.y < rect.y + h) 
			{
				return true;
			}
			
			return false;
		}
		
	});
});