define([
         "util/classutils",
         "howler"
         ],
         function(classUtils, howler) {
	
	"use strict";
	
	return classUtils.create({
		
		channel_id:'audio',
		howls:null,
		individual:true,
		android:false,
		current_individual_sound:null,
		
		init: function(sounds, channel_id, individual, play_if_android, is_android)
		{
			var self = this;
			
			if(channel_id != null)
				self.channel_id = channel_id;
			
			if(individual != null)
				self.individual = individual;
				
			if(play_if_android == false && is_android)
				self.android = true;
				
			self.howls = {};
			
			for(var i = 0; i < sounds.files.length; i++)
			{
				self.howls[sounds.files[i].id] = new howler.Howl({
					urls:[sounds.files[i].mp3, sounds.files[i].ogg],
					loop:sounds.loop,
					buffer:false
				});
			}
			
		},
		
		play: function(id, delay)
		{
			var self = this;
			
			if(self.android)
				return null;
			
			if(self.individual)
			{
				self.stop_all();
			}
			
			if(delay == null)
				delay = 0;
				
			setTimeout(function()
			{
				try
				{
					self.howls[id].play();
					
					if(self.individual)
						self.current_individual_sound = id;
				}
				catch(e)
				{
					console.log("Failed to play sound of id '" + id + "' on channel " + channel_id);
				}
			}, delay);
		},
		
		stop: function(id)
		{
			var self = this;
			
			if(self.android)
				return null;
			
			try
			{
				self.howls[id].stop();
			}
			catch(e)
			{
				console.log("Failed to stop sound of id '" + id + "' on channel " + channel_id);
			}
		},
		
		stop_all: function()
		{
			var self = this;
			
			if(self.android)
				return null;
			
			for(var i in self.howls)
			{
				self.howls[i].stop();
			}
		},
		
		mute: function()
		{
			var self = this;
			if (self.channel_id == "bgm")
			{
				for(var i in self.howls)
				{
					self.howls[i].mute();
				}	
			}
			
/* 			Howler.mute(); */
		},
		
		unmute: function()
		{
			var self = this;
			
			if (self.channel_id == "bgm")
			{
				for(var i in self.howls)
				{
					self.howls[i].unmute();
				}	
			}
			
/* 			Howler.unmute(); */
		}

	});
	
});