define([
         "util/classutils",
         "howler"
         ],
         function(classUtils, howler) {
	
	"use strict";
	
	return classUtils.create({
		
		channel_id:'audio',
		sound_sprite:null,
		howl:null,
		android:false,
		
		init: function(sound_sprite, channel_id, play_if_android, is_android)
		{
			var self = this;
			self.sound_sprite = sound_sprite;
			
			if(channel_id != null)
				self.channel_id = channel_id;
				
			if(play_if_android == false && is_android)
				self.android = true;
			
			var sprites = {};
			for(var i = 0; i < self.sound_sprite.sprites.length; i++)
			{
				sprites[self.sound_sprite.sprites[i].id] = [(self.sound_sprite.sprites[i].start * 1000), self.sound_sprite.sprites[i].duration * 1000];
			}

			self.howl = new howler.Howl({
				urls:[self.sound_sprite.mp3, self.sound_sprite.ogg],
				sprite:sprites,
				loop:self.sound_sprite.loop,
				buffer:false
			});
		},
		
		play: function(id, delay)
		{
			var self = this;
			
			if(self.android)
				return null;
			
			if(delay == null)
				delay = 0;
				
			setTimeout(function()
			{
				try
				{
					self.howl.play(id);
				}
				catch(e)
				{
					console.log("Failed to play sound of id '" + id + "' on channel " + channel_id);
				}
			}, delay);
		},
		
		play_random: function(sounds, random_int_function, delay)
		{
			var self = this;
			
			if(self.android)
				return null;
			
			self.play(sounds[random_int_function(0, sounds.length-1)], delay);
		},
		
		stop: function(id)
		{
			var self = this;
			
			if(self.android)
				return null;
			
			try
			{
				self.howl.stop(id);
			}
			catch(e)
			{
				console.log("Failed to stop sound of id '" + id + "' on channel " + channel_id);
			}
		},
		
		stop_all: function()
		{
			var self = this;
			
			if(self.android)
				return null;
			
			for(var i in self.sound_sprite.sprites)
			{
				self.howl.stop(self.sound_sprite.sprites[i].id);
			}
		},
		
		mute: function()
		{
			howler.Howler.mute();
		},
		
		unmute: function()
		{
			howler.Howler.unmute();
		}
		
		

	});
	
});